/**
 *****************************************************************************
   @file       AdcLib.h
   @brief      Set of ADC peripheral functions.

   @internal   002 @endinternal
   @version    V1.0
   @author     ADI
   @date       November 2015
   @par Revision History:
   - V0.1, June 2013: initial version.
   - V0.2, August 2013: added IDAC channels.
   - V0.3, January 2014: renamed PVDD_GAIN to PVDD_IDAC2
   - V0.4, November 2014: Added AdcDma
   - V1.0, November 2015: Changed Library format

   All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/
#ifdef __cplusplus
extern "C" {
#endif 

#include "DeviceHeader.h"

extern int AdcGo( int iStart);
extern int AdcRd(int iChan);
extern int AdcDma(int iEnable);
extern int AdcBuf( int iBufPDn, int iBufByp, int iRBufCfg);
extern int AdcSpeed(int iDiv);
extern int AdcPin( int iInN, int iInP);
extern int AdcCal( int iGain, int iOffset);
extern int AdcSeqCfg( int iEnable, int iChP, int iDelay);
extern int AdcSeqDiffCfg(int iDiff0,int iDiff2,int iDiff4,int iDiff6);
extern int AdcSeqStart( unsigned long iStart);
extern int AdcDigCompCfg( int iEnable, int iDir, int iThreshold);


//ADC Channel numbers
#define	AIN0		0
#define	AIN1		1
#define	AIN2		2
#define	AIN3		3
#define	AIN4		4
#define	AIN5		5
#define	AIN6		6
#define	AIN7		7
#define	AIN8		8
#define	AIN9		9
#define	AIN10		0xa
#define	AIN11		0xb
#define	AIN12		0xc
#define	AIN13		0xd
#define	AIN14		0xe
#define	AIN15		0xf
#define	VREFP_NADC	0x10
#define	VREFN_NADC	0x11
#define	AIN_IDAC3		0x12
#define	AGND		0x12
#define	AIN_IDAC1		0x13
#define	PGND		0x13
#define	AIN_IDAC0		0x14
#define	AIN_IDAC2		0x15
#define	TEMP_SENSOR	0x16
#define	VREFP_PADC	0x17
#define	PVDD_GAIN 0x18   //deprecated name
#define	PVDD_IDAC2 0x18
#define	IOVDD_2  0x19
#define	AVDD_2  0x1A
#define	VREFN_PADC	0x1B

//ADC sequencer Channel numbers
#define	AIN0_Seq		0x1
#define	AIN1_Seq		0x2
#define	AIN2_Seq		0x4
#define	AIN3_Seq		0x8
#define	AIN4_Seq		0x10
#define	AIN5_Seq		0x20
#define	AIN6_Seq		0x40
#define	AIN7_Seq		0x80
#define	AIN8_Seq		0x100
#define	AIN9_Seq		0x200
#define	AIN10_Seq   0x400
#define	AIN11_Seq	0x800
#define	AIN12_Seq	0x1000
#define	AIN13_Seq	0x2000
#define	AIN14_Seq	0x4000
#define	AIN15_Seq	0x8000
#define	IDAC3_Seq   0x40000
#define	IDAC1_Seq	0x80000
#define	IDAC0_Seq	0x100000
#define	IDAC2_Seq	0x200000
#define	TEMP_Seq	   0x400000
#define	VREFP_Seq	0x800000
#define	PVDD_Seq    0x1000000
#define	IOVDD_2_Seq 0x2000000
#define	AVDD_2_Seq  0x4000000
#define	VREFN_Seq	0x8000000


#ifdef __cplusplus
}
#endif 
