/**
 *****************************************************************************  
   @file       DacLib.h
   @brief      Set of DAC peripheral functions.
   - First configure DAC with DacCfg().
   - Set sync mode with DacSyn() if desired.
   - Output DAC value with DacWr().
   - Example:
      DacCfg(pADI_VDAC3,0,INT_REF);
      for(i1 = 0; i1<0x10000000; i1 += 0x1000000)
         DacWr(pADI_VDAC0,i1);

   @internal   002 @endinternal
   @version    V1.0
	@author     ADI
	@date       November 2015
   @par Revision History:
   - V0.1, May 2013: initial version.
   - V0.2, January 2013: removed DacBufCfg()
   - V1.0, November 2015: Changed library format


All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/
#ifdef __cplusplus
extern "C" {
#endif 

#include "DeviceHeader.h"


extern int DacCfg(ADI_VDAC_TypeDef *pPort, int iMode, int iRng);
extern int DacWr(ADI_VDAC_TypeDef *pPort, int iData);
extern int DacPwrDwn(ADI_VDAC_TypeDef *pPort, int iIcfg);

#define INT_REF 0x00
#define AVDD_REF 0x03
#define DACCON_MODE_12BIT  (0x0) 
#define DAC0CON_MODE_12BIT  (0x0) 
#define DAC1CON_MODE_12BIT  (0x0) 
#define DAC2CON_MODE_12BIT  (0x0) 
#define DAC3CON_MODE_12BIT  (0x0) 
#define DAC4CON_MODE_12BIT  (0x0) 
#define DAC5CON_MODE_12BIT  (0x0) 
#define DAC6CON_MODE_12BIT  (0x0) 
#define DAC7CON_MODE_12BIT  (0x0) 


