/** 
     @example  I2CMAster.c
     @brief I2C Master Basic democode. 
     Decription: Sends 5x bytes to the Slave when External IRQ0 (P0.3) is asserted
      - Interfaces to slave device via I2C0
      Can receive 5 bytes from Slave when XIRQ1 (P1.7) is asserted - will send these to UART
   @version  V0.2
   @author   ADI
   @date     December 2015
	 @par Revision History:
	 - V0.1, April 2013: initial version.
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h
	 

 *****************************************************************************


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>

#include <I2cLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <UrtLib.h>


// Function Definitions
void UartSetup(void);
void I2CMasterSetup(void);
unsigned char I2CWritePacket(unsigned int length);// Writes 5x Bytes to Slave
unsigned char I2CReadPacket( unsigned int length);// reads bytes back from Slave
void delay(long int);


volatile unsigned char ucValidUARTPacket = 0;     // Indicates a valid UART packet has been received.
volatile unsigned char ucTxBufferEmpty = 0;	     // Indicates UART Tx buffer empty
volatile unsigned char ucRdLength = 0;	           // Stores number of bytes to be read from Slave via I2C
volatile unsigned char StartByteGone = 0;
unsigned int ucBytesToRD = 0;			              // Stores number of bytes to be read from Slave via I2C
volatile unsigned int rxDat[256];	              // 256 byte buffer-array for I2C receive data
volatile unsigned char ucRxCount = 0;             // rxDat array control variable.
volatile unsigned char ucSendI2CPacket = 0;	     // Used in main loop to indicate I2C message is ready for transmission
volatile unsigned int ucRdCnt = 0;	              // Counter for UART receive messages
volatile unsigned char ucDataCnt = 0;             // Message Parser counter for UART messages
volatile unsigned char ucI2CWrCnt = 0;            // Counter for controlling I2C writes
volatile unsigned char ucTransmitInProgress =0;   // Indicates I2C Tx FIFO ready for data
volatile unsigned char ucReceiveInProgress = 0;   // Indicates I2C Rx sequence with the SLave is complete
volatile unsigned char ucMsgType = 0;             // Indicates I2CSerial.exe application requested I2C read or write
volatile unsigned int ucLength = 0;               // Indicates I2CSerial.exe "number of bytes" for I2C read or write
volatile unsigned int ucStartAddress = 0;	        // Indicates I2CSerial.exe "Start address" for I2C read or write
volatile unsigned char ucLoadCnt = 0;
volatile unsigned int uiTest = 0;
volatile unsigned char ucTransfer[512];	        // Buffer used during I2C "writes to Slave"	
volatile unsigned char ucI2CNT = 0;	              // The 4 following variables handle I2C reads of greater than 8 bytes
volatile unsigned char ucCOMSTA0 = 0;             // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;             // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;               // Variable used to read UART Rx buffer contents into 

int main (void)
{
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer resets 
   DioPul(pADI_GP0,0xFF);           // Enable pull-ups
	 DioPul(pADI_GP1,0xFF);           // Enable pull-ups
	 DioPul(pADI_GP2,0xFF);           // Enable pull-ups
	 DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   
   I2CMasterSetup();                              // Call function to initialize I2C master
   UartSetup();

    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 - P0.3 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
   // EiCfg(EXTINT1, INT_EN, INT_FALL);              // Enable External IRQ 1 - P1.7 
   // NVIC_EnableIRQ(EINT1_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
	DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);

   while (1)
   {
   if (ucSendI2CPacket == 1)                      // request received via UARt to send I2C Packet
		{
			if (ucMsgType == 0)                      // Write packet?
			{
			 	ucLength = 5;
            ucTransfer[0] = 0x41;
            ucTransfer[1] = 0x44;
            ucTransfer[2] = 0x75;
            ucTransfer[3] = 0x43;
            ucTransfer[4] = 0x4D;
            I2CWritePacket(ucLength);
				ucSendI2CPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
			}
			else                                     // read packet
			{
				ucLength = 5;
            I2CReadPacket(ucLength);
				ucSendI2CPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
				ucLoadCnt = 0;
			}
		}	
	}
}
// Function used to setup the I2C0 as master
void I2CMasterSetup(void)
{
      DioCfg(pADI_GP0,0x500);                     // Configure P0[4:7] as I2C pins     // Configure P0[4:7] as I2C pins

 //  I2cBaud(pADI_I2C0,0x62,0x63);                // Configure baud rate for 100khz
 //  I2cBaud(pADI_I2C1,0x62,0x63);
   I2cBaud(pADI_I2C0,0x13,0x15);                  // 400kHz Baud rate
   I2cMCfg(pADI_I2C0,0,
   I2CMCON_IENCMP_EN|
   I2CMCON_IENACK_EN|
   I2CMCON_IENALOST_EN|
   I2CMCON_IENMTX_EN|
   I2CMCON_IENMRX_EN,
   I2CMCON_MASEN_EN);                             // Enable I2C0 master mode. Enable Interrupt sources
   NVIC_EnableIRQ(I2C0M_IRQn);                    // Enable i2C0 Master interrupt source
}
// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                     // Enable Rx, Tx and Rx buffer full Interrupts
   pADI_UART->COMLCR = 0x3;	
   NVIC_EnableIRQ(UART_IRQn);                     // Enable UART interrupt source in NVIC
}
// This function Starts an I2C Write sequence
unsigned char I2CWritePacket(unsigned int length )
{
	// Transmit
	ucTransmitInProgress = 1;
	I2cFifoFlush(pADI_I2C0,MASTER,ENABLE);	        // Flush Master Tx FIFO
	I2cFifoFlush(pADI_I2C0,MASTER,DISABLE);
   I2cTx(pADI_I2C0,MASTER,ucTransfer[0]);         // send i2c byte - register Address 0x7  
   I2cMWrCfg(pADI_I2C0,0xA0);                     // set i2c address	 (LSB = 0, Master Write)	
    	
	ucI2CWrCnt = 1;
	while (ucTransmitInProgress == 1)
	{}
	
	return 1;
}
// Function used to commence I2C Read sequence
unsigned char I2CReadPacket( unsigned int length)
{
	unsigned char n = 0;
	
	ucBytesToRD = ucLength - 1;;
	// Transmit
	ucTransmitInProgress = 1;
	StartByteGone = 1;
	I2cFifoFlush(pADI_I2C0,MASTER,ENABLE);	        // Flush Master Tx FIFO
	I2cFifoFlush(pADI_I2C0,MASTER,DISABLE); 

   pADI_I2C0->I2CMRXCNT = ucLength;               // Load RXCNT register with required number of bytes to read from Slave
   I2cTx(pADI_I2C0,MASTER,ucStartAddress);        // send i2c byte - register Address  
   pADI_I2C0->I2CADR0	= 0xA1;                   // Write to i2C address register and set read bit

	ucLength = 0;	
	ucReceiveInProgress = 1;

	while (ucReceiveInProgress == 1) {}            // Wait for I2C packet to be recieved - flag cleared in I2C0 read IRQ handler
	 
	ucRdLength = 0;
	ucDataCnt = 0;
	ucRdCnt = 0;
	ucTxBufferEmpty = 0;                           // Start forwarding packet UART
	pADI_UART->COMTX = rxDat[0];
	ucSendI2CPacket = 0;
	for (n = 1; n < ucBytesToRD +1 ; n++)          // Send received packet to the PC via UART
	{
		while (ucTxBufferEmpty != 1){}
		ucTxBufferEmpty = 0;
		pADI_UART->COMTX = rxDat[n];
	}
	return 1;
}

void Ext_Int0_Handler()
{
   ucSendI2CPacket = 1;
  ucMsgType = 0;
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source

}
/* void Ext_Int1_Handler()
{
    ucSendI2CPacket = 1;
   ucMsgType = 1;
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x2;                   // Clear the interrupt1 source

} */
void I2C0_Master_Int_Handler()
{
   unsigned int I2CMSTATUS = 0;
   
   I2CMSTATUS = I2cSta(pADI_I2C0,MASTER);
    // Transmit
	if((I2CMSTATUS & 0x4) == 0x4)  	              // Master Transmit IRQ	
	{											    
		if (ucI2CWrCnt < ucLength)
      {                                           // Tx FIFO ready for another byte
         I2cTx(pADI_I2C0,MASTER,ucTransfer[ucI2CWrCnt++]);	
      }
		else 
		{
			ucTransmitInProgress = 0;
			ucRxCount = 0; 
			ucI2CNT = 0;
		}
   }			
	// Receive
	if((I2CMSTATUS & 0x8) == 0x8)				        // Master Recieve IRQ
	{
		rxDat[ucRxCount++] = I2cRx(pADI_I2C0,MASTER);
		if(ucRxCount == (ucBytesToRD+1))
			ucReceiveInProgress = 0;	
		ucI2CNT++;
   }
   if((I2CMSTATUS & 0x200) == 0x200)			     // Rx overflow
	{
		uiTest++;								
	}
	if((I2CMSTATUS & 0x100) == 0x100)			     // Transmission Complete/Stop detected
	{
		uiTest++;								
	}
	if((I2CMSTATUS & 0x80) == 0x80)			        // NACK Data detected
	{
		uiTest++;								
	}	
   if((I2CMSTATUS & 0x40) == 0x40)			        // NACK data detected
	{
		uiTest++;							
   }
	if((I2CMSTATUS & 0x10) == 0x10)			        // NACK Address detected
	{
		uiTest++;							
   }
}
void I2C1_Master_Int_Handler()
{
   
}
void UART_Int_Handler()
{  	
      ucCOMSTA0 = UrtLinSta(pADI_UART);
      ucCOMIID0 = UrtIntSta(pADI_UART);
		if ((ucCOMIID0 & 0x2) == 0x2)	              // Transmit buffer empty
		{
		  ucTxBufferEmpty = 1;
		}
		
		if ((ucCOMIID0 & 0x4) == 0x4)	              // Receive byte
		{

		}
}


// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




