/** 
     @example  I2C0 master for EEPROM interface .c
    @brief I2C Master EEPROM democode. Decription: interfaces to PC via UART link. 
      - Interfaces to slave device via I2C0
   @version  V0.2
   @author   ADI
   @date     March 2013 
	  -  V0.2:updated November 2014 to support Auto stretch mode

 *****************************************************************************


              
All files for ADuCM320i provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <ADuCM320i.h>

#include <I2cLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <UrtLib.h>

void delay(long int);
// Function Definitions
void Uart_Packet_Handler( volatile unsigned int value, volatile unsigned char Byte_Number);
unsigned char I2CWritePacket( unsigned int length);
unsigned char I2CReadPacket( unsigned int length);
void I2CMasterSetup(void);
void UartSetup(void);                             // UART Setup function

volatile unsigned char ucValidUARTPacket = 0;     // Indicates a valid UART packet has been received.
volatile unsigned char ucTxBufferEmpty = 0;	     // Indicates UART Tx buffer empty
volatile unsigned char ucRdLength = 0;	           // Stores number of bytes to be read from Slave via I2C
volatile unsigned char StartByteGone = 0;
unsigned int ucBytesToRD = 0;			              // Stores number of bytes to be read from Slave via I2C
volatile unsigned int rxDat[256];	              // 256 byte buffer-array for I2C receive data
volatile unsigned char ucRxCount = 0;             // rxDat array control variable.
volatile unsigned char ucSendI2CPacket = 0;	     // Used in main loop to indicate I2C message is ready for transmission
volatile unsigned int ucRdCnt = 0;	              // Counter for UART receive messages
volatile unsigned char ucDataCnt = 0;             // Message Parser counter for UART messages
volatile unsigned char ucI2CWrCnt = 0;            // Counter for controlling I2C writes
volatile unsigned char ucTransmitInProgress =0;   // Indicates I2C Tx FIFO ready for data
volatile unsigned char ucReceiveInProgress = 0;   // Indicates I2C Rx sequence with the SLave is complete
volatile unsigned char ucMsgType = 0;             // Indicates I2CSerial.exe application requested I2C read or write
volatile unsigned int ucLength = 0;               // Indicates I2CSerial.exe "number of bytes" for I2C read or write
volatile unsigned int ucStartAddress = 0;	        // Indicates I2CSerial.exe "Start address" for I2C read or write
volatile unsigned char ucLoadCnt = 0;
volatile unsigned int uiTest = 0;
volatile unsigned char ucTransfer[512];	        // Buffer used during I2C "writes to Slave"	
volatile unsigned char ucI2CNT = 0;	              // The 4 following variables handle I2C reads of greater than 8 bytes
volatile unsigned char ucCOMSTA0 = 0;             // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;             // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;               // Variable used to read UART Rx buffer contents into 

int main (void)
{
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer  
   I2CMasterSetup();                              // Call function to initialize I2C master
   UartSetup();                                   // Call function to initialize UART
    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC

	DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);

   while (1)
   {
   if (ucSendI2CPacket == 1)                      // request received via UARt to send I2C Packet
		{
			if (ucMsgType == 0)                      // Write packet?
			{
			 	I2CWritePacket(ucLength);
				ucSendI2CPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
			}
			else                                     // read packet
			{
				I2CReadPacket(ucLength);
				ucSendI2CPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
				ucLoadCnt = 0;
			}
		}	
	}
}
// Function used to setup the I2C0 as master
void I2CMasterSetup(void)
{
    DioCfg(pADI_GP0,0x500);                     // Configure P0[4:7] as I2C pins     // Configure P0[4:7] as I2C pins
    I2cAutoStretch(pADI_I2C0,MASTER,STRETCH_EN,1);// Enable Master Auto clock stretching
 //  I2cBaud(pADI_I2C0,0x62,0x63);                // Configure baud rate for 100khz
 //  I2cBaud(pADI_I2C1,0x62,0x63);
   I2cBaud(pADI_I2C0,0x13,0x15);                  // 400kHz Baud rate
   I2cMCfg(pADI_I2C0,0,
   I2CMCON_IENCMP_EN|
   I2CMCON_IENACK_EN|
   I2CMCON_IENALOST_EN|
   I2CMCON_IENMTX_EN|
   I2CMCON_IENMRX_EN,
   I2CMCON_MASEN_EN);                             // Enable I2C0 master mode. Enable Interrupt sources
   NVIC_EnableIRQ(I2C0M_IRQn);                    // Enable i2C0 Master interrupt source
}
// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate	
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                     // Enable Rx, Tx and Rx buffer full Interrupts
   pADI_UART->COMLCR = 0x3;	
   NVIC_EnableIRQ(UART_IRQn);                     // Enable UART interrupt source in NVIC
}
// This function Starts an I2C Write sequence
unsigned char I2CWritePacket(unsigned int length )
{
	// Transmit
	ucTransmitInProgress = 1;
	I2cFifoFlush(pADI_I2C0,MASTER,ENABLE);	        // Flush Master Tx FIFO
	I2cFifoFlush(pADI_I2C0,MASTER,DISABLE);
   I2cMWrCfg(pADI_I2C0,0xA0);                     // set i2c address	 (LSB = 0, Master Write)	
   I2cTx(pADI_I2C0,MASTER,ucStartAddress);        // send i2c byte - register Address 0x7   
	
	ucI2CWrCnt = 0;
	while (ucTransmitInProgress == 1)
	{}
	
	return 1;
}
// Function used to commence I2C Read sequence
unsigned char I2CReadPacket( unsigned int length)
{
	unsigned char n = 0;
	
	ucBytesToRD = ucLength - 1;;
	// Transmit
	ucTransmitInProgress = 1;
	StartByteGone = 1;
	I2cFifoFlush(pADI_I2C0,MASTER,ENABLE);	        // Flush Master Tx FIFO
	I2cFifoFlush(pADI_I2C0,MASTER,DISABLE); 

	NVIC_DisableIRQ(I2C0M_IRQn);                   // Disable I2C0 Master interrupt source

	// Repeated Start Condition!
   pADI_I2C0->I2CMRXCNT = ucLength;               // Load RXCNT register with required number of bytes to read from Slave
   I2cTx(pADI_I2C0,MASTER,ucStartAddress);        // send i2c byte - register Address  
   I2cMWrCfg(pADI_I2C0,0xA0);                     // set i2c address	 (LSB = 0, Master Write)
   while ((pADI_I2C0->I2CMSTA  & 0x3) != 0x0)     // Wait for Tx FIFO to empty
   {}
   pADI_I2C0->I2CADR0	= 0xA1;                   // Write to i2C address register and set read bit
   while ((pADI_I2C0->I2CMSTA  & 0x4) != 0x0) {}  // wait for master transmit to complete before re-enabling I2C Interrupts
	NVIC_EnableIRQ(I2C0M_IRQn);                    // Enable i2C0 Master interrupt source

	ucLength = 0;	
	ucReceiveInProgress = 1;

	while (ucReceiveInProgress == 1) {}            // Wait for I2C packet to be recieved - flag cleared in I2C0 read IRQ handler
	 
	ucRdLength = 0;
	ucDataCnt = 0;
	ucRdCnt = 0;
	ucTxBufferEmpty = 0;                           // Start forwarding packet UART
	pADI_UART->COMTX = rxDat[0];
	ucSendI2CPacket = 0;
	for (n = 1; n < ucBytesToRD +1 ; n++)          // Send received packet to the PC via UART
	{
		while (ucTxBufferEmpty != 1){}
		ucTxBufferEmpty = 0;
		pADI_UART->COMTX = rxDat[n];
	}
	return 1;
}
// This Function Parses the UART packets received from the PC application.
void Uart_Packet_Handler( volatile unsigned int value, volatile unsigned char Byte_Number)
{
  switch(Byte_Number)
  {
	   case 1:	                                   // Read/Write indicator
	   	{
			ucMsgType = value;
			break;
		} 
	   case 2:                                     //	Length of data in packet
	   {
	   		ucLength = value;
			break;
	   }	   
	   case 6 :                                    // Memory Start Address
	   {
	   		ucStartAddress = value;
			break;
	   }
	   case 9 :                                    // Memory Start Address
	   {
	   		if (ucMsgType == 1)	                 // if msg type is a read - reset buffer
			{
				ucDataCnt = 0;	                       // Reset message parse register
				ucSendI2CPacket = 1;                  //Send Packet to the slave
				ucRdCnt = 0;
				ucValidUARTPacket = 0;
			}
			break;
	   }
		default :
	   break;
	}
}
void Ext_Int0_Handler()
{
  
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source
   pADI_UART->COMTX = 0x55;
}
void I2C0_Master_Int_Handler()
{
   unsigned int I2CMSTATUS = 0;
   
   I2CMSTATUS = I2cSta(pADI_I2C0,MASTER);
    // Transmit
	if((I2CMSTATUS & 0x4) == 0x4)  	              // Master Transmit IRQ	
	{											    
		if (ucI2CWrCnt < ucLength)
      {                                           // Tx FIFO ready for another byte
         I2cTx(pADI_I2C0,MASTER,ucTransfer[ucI2CWrCnt++]);	
      }
		else 
		{
			ucTransmitInProgress = 0;
			ucRxCount = 0; 
			ucI2CNT = 0;
		}
   }			
	// Receive
	if((I2CMSTATUS & 0x8) == 0x8)				        // Master Recieve IRQ
	{
		rxDat[ucRxCount++] = I2cRx(pADI_I2C0,MASTER);
		if(ucRxCount == (ucBytesToRD+1))
			ucReceiveInProgress = 0;	
		ucI2CNT++;
   }
   if((I2CMSTATUS & 0x200) == 0x200)			     // Rx overflow
	{
		uiTest++;								
	}
	if((I2CMSTATUS & 0x100) == 0x100)			     // Transmission Complete/Stop detected
	{
		uiTest++;								
	}
	if((I2CMSTATUS & 0x80) == 0x80)			        // NACK Data detected
	{
		uiTest++;								
	}	
   if((I2CMSTATUS & 0x40) == 0x40)			        // NACK data detected
	{
		uiTest++;							
   }
	if((I2CMSTATUS & 0x10) == 0x10)			        // NACK Address detected
	{
		uiTest++;							
   }
}
void I2C1_Master_Int_Handler()
{
   
}
void UART_Int_Handler()
{  	
      ucCOMSTA0 = UrtLinSta(pADI_UART);
      ucCOMIID0 = UrtIntSta(pADI_UART);
		if ((ucCOMIID0 & 0x2) == 0x2)	              // Transmit buffer empty
		{
		  ucTxBufferEmpty = 1;
		}
		
		if ((ucCOMIID0 & 0x4) == 0x4)	              // Receive byte
		{
			ucComRx	= UrtRx(pADI_UART);
			if 	((ucRdCnt == 0) && (ucComRx == 8)) // Received Start character 0x08
				ucValidUARTPacket = 1;
			if ((ucRdCnt < 10) && (ucSendI2CPacket == 0) && (ucValidUARTPacket == 1))
			{					
				Uart_Packet_Handler( ucComRx, ucRdCnt);
				ucRdCnt++;
			}
			else if ((ucSendI2CPacket == 0) && (ucValidUARTPacket == 1))
			{
				ucTransfer[ucDataCnt++] = UrtRx(pADI_UART);				 
				if (ucDataCnt == ucLength)
				{
			  		ucDataCnt = 0;	                    // Reset message parse register
					ucSendI2CPacket = 1;               //Send Packet to the slave
					ucValidUARTPacket = 0;
				}
			}
         if ((ucCOMSTA0 & 0x2) == 0x2)
         {
            uiTest++;
         }
		}
}


// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




