/** 
     @example  I2C0 Slave for EEPROM interface.c
    @brief I2C Slave EEPROM democode. Interfaces to master via I2C0 
      - Implements 256 byte EEPROM with I2C interface
   @version  V0.2
   @author   ADI
   @date     March 2013 
     - updated November 2014 to support Auto stretch mode
 *****************************************************************************


              
All files for ADuCM320i provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <ADuCM320i.h>

#include <I2cLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>

void delay(long int);
// Function Definitions
void delay(long int);                             // delay function
void I2CSlaveSetup(void);                         // I2C Slave setup function 

unsigned char byte_addr0 = 0x00;                  // Used to address EEPROM address
unsigned char  wr_byte_addr0 = 0;                 // Write address pointer to EEPROM

unsigned char  A0h[256], A2h[256];                // Arrays implementing EEPROM
unsigned char  i2c_first = 1;                     // Staus flag for incoming I2C packet

unsigned char ucTxCnt = 0;                        // Counts number of I2C bytes transmitted
unsigned char ucRxCnt =0;                         // Counts number of I2C bytes received
unsigned char ucEmptyCnt = 0;
unsigned char ucStopCnt =0;
unsigned char ucTestExtraRead = 0;
unsigned char ucFirstDataWrite = 0;

int main (void)
{
   unsigned int i;
   
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer  
   I2CSlaveSetup();                               // Call function to initialize I2C master

	DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);
  // Initialize table    
    for(i=0;i<256;i++)                            // Initialize memory values to 0
   {
        A0h[i]=i;
        A2h[i]=(255-i);
   }
   i2c_first = 1;
   I2cFifoFlush(pADI_I2C0,SLAVE,ENABLE);          // Flush the I2C FIFOs on startup
   while (1)
   {
      pADI_GP2->GPTGL = 0x10;                     // Toggle P2.4
      delay(0x60000);                             // Delay routine
	}
}
// Function used to setup the I2C0 as Slave
void I2CSlaveSetup(void)
{
  I2cAutoStretch(pADI_I2C0,SLAVE,STRETCH_EN,1);    // Enable Slave Auto clock stretching  
	DioCfg(pADI_GP0,0x500);                       // Configure P0[5:4] as I2C pins   
    DioPul(pADI_GP0,0xFF);                        // Enable Pull-ups on P0rt 0 bits
   I2C0SIDCfg(0xA0,0xA6,0xA8,0xA0);               // Configure Slave ID value;           
    I2cFifoFlush(pADI_I2C0,SLAVE,ENABLE);         // Enable flush of Slave FIFOs
   I2cFifoFlush(pADI_I2C0,SLAVE,DISABLE);
   I2cSCfg(pADI_I2C0,0,
   I2CSCON_IENREPST_EN|I2CSCON_IENSTX_EN|
   I2CSCON_IENSRX_EN|I2CSCON_IENSTOP_EN,
   I2CSCON_SLVEN_EN);                             // Enable Slave mode - all interrupt sources
   NVIC_EnableIRQ(I2C0S_IRQn);                    // Enable I2C0 Slave interrupt source
}

// I2C0 Slave Interrupt Handler
void I2C0_Slave_Int_Handler()
{
   unsigned int I2CSSTATUS = 0;
   volatile unsigned char ucTest = 0;
   I2CSSTATUS = I2cSta(pADI_I2C0,SLAVE);          // read I2C0 Slave status
   if ((I2CSSTATUS & BIT3) == BIT3)               // If I2C Slave Rx IRQ
	 {
	 		if(i2c_first == 1)                       // first slave-receiver INT?
			{   
			   byte_addr0 = I2cRx(pADI_I2C0,SLAVE);  // get byte address
			   wr_byte_addr0 = byte_addr0; 
			   I2cTx(pADI_I2C0,SLAVE,A0h[byte_addr0]);// Load Slave Tx FIFO
			   byte_addr0++;                
			   i2c_first = 0;
			   ucFirstDataWrite = 1;
		 	}
		   else
		 	{
				if (ucFirstDataWrite == 1)
				{
				 	 byte_addr0 -= 1;
				}
			    A0h[wr_byte_addr0] = I2cRx(pADI_I2C0,SLAVE);// get byte data
	          byte_addr0 = A0h[wr_byte_addr0];     // get byte address
			    byte_addr0++;						        // increment address
				wr_byte_addr0++;
			    ucFirstDataWrite = 0;                       
		    }
	   }
       if ((I2CSSTATUS & 0x4) == 0x4)             // If I2C Master Tx IRQ
	   {
			I2cTx(pADI_I2C0,SLAVE,A0h[byte_addr0++]);// Load Tx FIFO
      }   
      if ((I2CSSTATUS & 0x2) == 0x2)              // Slave Transmit underflow
     {
        ucTest++;
     }
      if ((I2CSSTATUS & 0x10) == 0x10)            // Slave receive overflow
     {
        ucTest++;
     }
	  if ((I2CSSTATUS & 0x20) == 0x20)             // If I2C NOACK
     {
        ucTest++;
     }
     if ((I2CSSTATUS & 0x80) == 0x80)             // I2C General Call Interrupt
     {

     }
	   if ((I2CSSTATUS & 0x400) == 0x400)          // If I2C Stop after Start condition detected IRQ
	   {
		  	i2c_first = 1;
			I2cFifoFlush(pADI_I2C0,SLAVE,ENABLE);    // Enable flush of Slave FIFOs
         I2cFifoFlush(pADI_I2C0,SLAVE,DISABLE);
			ucStopCnt++; 
	   }
	   if ((I2CSSTATUS & 0x2000) == 0x2000)        // If I2C Repeated Start detected IRQ
	   {
			i2c_first = 1;	                          // Reset counter Array pointer.
	   }
}


// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




