/**
 *****************************************************************************
   @example  nvic.c
   @brief    Demonstrate the use of NVIC Priorities
             EIRQ0 (P0.3) is the lowest priority, Timer 0 is the next priority
             and EIRQ1 (P1.7) is the highest priority.
             When EIRQ0 is triggered it never leaves the handler function, 
             but Timer 0 still works as it has a higher priority. When EIRQ1 
             is triggered no other interrupts occur as it's the highest priority
             
   @version  V1.0
   @author   ADI
   @date     December 2015
	 @par Revision History:
	 - V0.1, April 2013: initial version.
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

              
All files for the device provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>
#include <WdtLib.h>
#include <GptLib.h>
#include <IntLib.h>
#include <DioLib.h>


int test;

int main (void)
{
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);                                 // Disable the Watchdog timer resets
   
   /* Configure Timer 0 */
   GptLd(pADI_TMR0,0x1000);                                                 // Load timer 0
   GptCfg(pADI_TMR0,TCON_CLK_HCLK,
   TCON_PRE_DIV1,TCON_MOD_PERIODIC|TCON_ENABLE);                            // Enable Timer1 for periodic mode using HCLK as clock source
   NVIC_EnableIRQ(TIMER0_IRQn);                                             // Enable Timer 0  interrupt source in NVIC
   NVIC_SetPriority(TIMER0_IRQn, 2);
   test = NVIC_GetPriority(TIMER0_IRQn);
   
    /* Configure External Interrupt 0  P0.3*/

   EiCfg(EXTINT0, INT_EN, INT_LOW);                                         // Enable External IRQ0 
   NVIC_EnableIRQ(EINT0_IRQn);                                              // Enable External IRQ0 interrupt source in NVIC
   NVIC_SetPriority(EINT0_IRQn, 3);                                        
   test = NVIC_GetPriority(EINT0_IRQn);
   
   /* Configure External Interrupt 0  P1.7*/
 
   EiCfg(EXTINT1, INT_EN, INT_LOW);                                         // Enable External IRQ1 
   NVIC_EnableIRQ(EINT1_IRQn);                                              // Enable External IRQ1 interrupt source in NVIC
   NVIC_SetPriority(EINT1_IRQn, 1);                                         // 
   test = NVIC_GetPriority(EINT1_IRQn);
   
   
   DioOenPin(pADI_GP1,PIN0,1);                                              // Configure P1.0 as an output - to toggle with EIRQ0
   DioClr(pADI_GP1, BIT0);
   
   DioOenPin(pADI_GP1,PIN1,1);                                              // Configure P1.1 as an output - to toggle with EIRQ1
   DioSet(pADI_GP1, BIT1);

	DioOenPin(pADI_GP1,PIN2,1);                                              // Configure P1.2 as an output - to toggle with Timer 0
   DioSet(pADI_GP1, BIT2);

   while (1)
   {
  
	}
}

void GP_Tmr0_Int_Handler()
{
   volatile int uiT0STA = 0;
   
   uiT0STA = GptSta(pADI_TMR0);
   pADI_TMR0->TCLRI = 0x1; // Clear timer interrupt
   DioTgl(pADI_GP1, BIT2);                                             
}


void Ext_Int0_Handler()
{
  
         
   pADI_INTERRUPT->EICLR = 0x1;   // Clear the interrupt0 source
   while(1){DioTgl(pADI_GP1, BIT0); }
   
}

void Ext_Int1_Handler()
{
      
   pADI_INTERRUPT->EICLR = 0x2;   // Clear the interrupt0 source
   while(1){DioTgl(pADI_GP1, BIT1); }

   
}




