/**
 *****************************************************************************
   @example PWM.c
   @brief   The PWM is configured to operate in standard mode.
   - PWM0 on P1.2 (mode 1)
   - PWM1 on P1.3 (mode 1)
   - PWM2 on P1.4 (mode 1)
   - PWM3 on P1.5 (mode 1)
   - PWM4 on P1.6 (mode 1)
   - PWM5 on P1.7 (mode 1)
   - PWMTRIP on P2.0 (mode 1)

   @version  V0.2
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, June 2013: Initial version
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


#include <WdtLib.h>
#include <GptLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <PwmLib.h>
volatile unsigned int uiTime0 = 0;	              // Used for reading return value from PWMTime function
volatile unsigned int uiTime1 = 0;	              // Used for reading return value from PWMTime function
volatile unsigned int uiTime2 = 0;	              // Used for reading return value from PWMTime function
volatile unsigned int uiTime3 = 0;	              // Used for reading return value from PWMTime function
volatile int Error= 0;

int main (void)
{
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer resets
   pADI_CLKCTL->CLKCON5 = 0;                      // Enable clock to all peripherals
   DioCfg(pADI_GP1,0x5550);                       // Configure P1[2:7] as PWM pins     
   DioCfg(pADI_GP2,0x0001);                       // Configure P2[0] as PWM pin    
   PwmInit(HCLK_2,PWMCON0_PWMIEN_EN,
   PWMCON0_SYNC,0);                               // Enable HCLK/2 as PWM clock and all PWM IRQ sources
   
   uiTime0 = PwmTime(PWM0_1,200,150,40);          // 200kHz frequency, PWM0 output for 75% high. PWM1 output 20% duty cycle.
   uiTime1 = PwmTime(PWM2_3,400,350,100);         // 100kHz frequency PWM2 output for 87.5% high. PWM3 output 25% dutycycle.
   uiTime2 = PwmTime(PWM4_5,800,400,399);         // 50kHz freq. PWM4 output for 50% high. PWM5 output 49.8% dutycycle.
 
   NVIC_EnableIRQ(PWM_TRIP_IRQn);			        // Enable PWM Trip IRQ
   NVIC_EnableIRQ(PWM_PAIR0_IRQn);			        // Enable pair 0 IRQ
   NVIC_EnableIRQ(PWM_PAIR1_IRQn);			        // Enable pair 1 IRQ
   NVIC_EnableIRQ(PWM_PAIR2_IRQn);			        // Enable pair 2 IRQ   
    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC

  if (uiTime0 != 1)
		Error = 1;                                  // Error with PWM pair 0 
   if (uiTime1 != 1)
		Error = 1;                                  // Error with PWM pair 1
   if (uiTime2 != 1)
		Error = 1;                                  // Error with PWM pair 2
   if (uiTime3 != 1)
		Error = 1;                                  // Error with PWM pair 3	
   PwmGo(PWMCON0_PWMEN_EN,0);                     // Enable PWM outputs
   while (1)
   {
	
   }
}
void PWMTrip_Int_Handler ()
{           
   PwmClrInt(PWMICLR_TRIP_EN);                      // Clear PWM trip IRQ
}
void PWM0_Int_Handler()
{
   PwmClrInt(PWMICLR_PWM0_EN);
}
void PWM1_Int_Handler ()
{
   PwmClrInt(PWMICLR_PWM1_EN);
}
void PWM2_Int_Handler()
{
   PwmClrInt(PWMICLR_PWM2_EN);
}


void Ext_Int0_Handler()
{
  
   pADI_GP2->GPTGL = 0x10;        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;   // Clear the interrupt0 source
   
}


// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}

