/**
 *****************************************************************************
   @example  SPIDMA_Slave.c
   @brief    SPI1 slave DMA example
      After 32 bytes are received from the master an interrupt is generated
      and the DMA is set up again for another 16 transfers

   @version  V0.3
   @author   ADI
   @date     December 2015 
   @par Revision History:
   - V0.1, July 2013: Initial release
   - V0.2, May 2014: Added comment about disabling SPI DMA
	 - V0.3, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

   
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.


**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


//#include <ClkLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <WdtLib.h>
#include <DioLib.h>
#include <DmaLib.h>
#include <SpiLib.h>


void delay(long int);
void SPI1INIT(void);
void DMAINIT(void);

int dma_done = 0;

unsigned char ucIrqCnt = 0;                              // Used to count number of SPI1 DMA interrupts
unsigned char ucSPIDMAEnable = 0;                        // Flag used to setup 2nd and subsequent transfers
unsigned char uxSPI1WrData[]={0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
                              0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F};
unsigned char uxSPI1RdData[64];

unsigned short int dataBlk[12];    // Array of tx data


int main (void)
{
   unsigned int numDmaTran = 0;
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);                                 // Disable the Watchdog timer resets
   
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
	 //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   DMAINIT();                                                                 // Initialize DMA controller
   SPI1INIT();                                                                // Initialize SPI1 block 
   
   NVIC_EnableIRQ(DMA_SPI1_RX_IRQn);                                          // SPI1 Tx DMA interrupt enable
     
   numDmaTran = 16;
   DmaStructPtrInSetup(SPI1RX_C,numDmaTran,uxSPI1RdData);   // Setup source/destination pointer fields
   DmaGo(SPI1RX_C,numDmaTran,DMA_BASIC);                    // Enable SPI1RX DMA structure
   DmaStructPtrOutSetup(SPI1TX_C,numDmaTran,uxSPI1WrData);  // Setup source/destination pointer fields
   DmaGo(SPI1TX_C,numDmaTran,DMA_BASIC);                    // Enable SPI1RX DMA structure
   DmaSet(0,SPI1TX_B|SPI1RX_B,0,0);                         // Enable DNA structures for SPI1 TX and RX
   SpiCount(pADI_SPI1, numDmaTran*2, 1);                    // Number of bytes the SPI should receive (2* number of DMA transfers)
   SpiDma(pADI_SPI1,SPIDMA_IENRXDMA_EN,SPIDMA_IENTXDMA_EN,SPIDMA_ENABLE_EN);  // Enable SPI1 DMA operation - Tx & Rx
   
   while (1)
   {
   DioTgl(pADI_GP2,0x10);     // Toggle P2.4
   delay(0x60000);            // Delay routine
      if (dma_done ==1)
      {
         dma_done = 0;
         for (i = 0; i<32; i++){
            uxSPI1WrData[i]++;
         }
         DmaStructPtrInSetup(SPI1RX_C,numDmaTran,uxSPI1RdData);   // Setup source/destination pointer fields
         DmaGo(SPI1RX_C,numDmaTran,DMA_BASIC);                    // Enable SPI1RX DMA structure
         DmaStructPtrOutSetup(SPI1TX_C,numDmaTran,uxSPI1WrData);  // Setup source/destination pointer fields
         DmaGo(SPI1TX_C,numDmaTran,DMA_BASIC);                    // Enable SPI1RX DMA structure

         DmaSet(0,SPI1TX_B|SPI1RX_B,0,0);
         SpiCount(pADI_SPI1, numDmaTran*2, 1);

         SpiDma(pADI_SPI1,SPIDMA_IENRXDMA_EN,SPIDMA_IENTXDMA_EN,SPIDMA_ENABLE_EN);
      }
   }
}

void SPI1INIT(void)
{                    
   DioCfgPin(pADI_GP1, PIN4, 2);                               // Configure P1[7:4] for SPI1
   DioCfgPin(pADI_GP1, PIN5, 2);
   DioCfgPin(pADI_GP1, PIN6, 2);
   DioCfgPin(pADI_GP1, PIN7, 2);

   SpiCfg(pADI_SPI1,SPICON_MOD_TX1RX1,SPICON_MASEN_DIS,
      SPICON_CON_DIS|SPICON_ENABLE_EN|SPICON_OEN_EN);          // Enable SPI1 master mode, continuous transfer, start on write to Tx
   
   DmaPeripheralStructSetup(SPI1RX_C,DMA_DSTINC_HWORD|
      DMA_SRCINC_NO|DMA_SIZE_HWORD);                           // Setup SPI1 Rx DMA structure
   DmaPeripheralStructSetup(SPI1TX_C,DMA_DSTINC_NO|
      DMA_SRCINC_HWORD|DMA_SIZE_HWORD);                        // Setup SPI1 Tx DMA structure
}
void DMAINIT(void)  
{
   DmaBase();                 
}
// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}
void WakeUp_Int_Handler(void)
{
  
}
void Ext_Int2_Handler ()
{           

}
void Ext_Int0_Handler ()
{         
   EiClr(EXTINT0);                                          // Clear EINT0 interrupt flag
}

void UART_Int_Handler ()
{

} 
void DMA_SPI1_RX_Int_Handler()
{
   // After the DMA interrupt occurs and before setting up the DMA for the next SPI sequence we need to disable
   // the SPI DMA mode
  SpiDma(pADI_SPI1,SPIDMA_IENRXDMA_DIS,SPIDMA_IENTXDMA_DIS,SPIDMA_ENABLE_DIS);
  dma_done = 1;
}



