/**  
*****************************************************************************
   @example UartExample.c
   @brief   Sends string to PC at 230K baud
    LED toggles and string sent to UART for every IRQ0 (push button IRQ)

   @version V0.4
   @author  ADI
   @date    December 2015 
   @par Revision History:
   - V0.1, April 2013: initial version.
   - V0.2, January 2014: Some parameters passed to UrtCfg() were later overwritten
	 - V0.3, Enabled GPIO pull-ups which fixes the XIRQ not being triggered some of the time
	 - V0.4, December 2015: Changed ADuCM3xx.h to DeviceHeader.h


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


#include <WdtLib.h>
#include <GptLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <UrtLib.h>

void UartSetup(void);                             // UART Setup function
void TestButtonPress(void);                       // Sends string to UART when IRQ0 asserted
volatile unsigned char ucCOMSTA0 = 0;             // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;             // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;               // Variable used to read UART Rx buffer contents into 
unsigned char ucTxBufferEmpty  = 0;		  // Used to indicate that the UART Tx buffer is empty
unsigned char szTemp[64] = "";		          // Used to store ADC1 result before printing to UART
unsigned char ucButtonPress = 0;                  // used to detect IRQ0 interrupt
int main (void)
{
   int i,j,k = 0;
   unsigned char nLen = 0; 
  
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);        // Disable the Watchdog timer resets
   pADI_CLKCTL->CLKCON5 = 0;                      // Enable clock to all peripherals
   DioPul(pADI_GP0, 0xff);
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED 
   DioSet(pADI_GP2, BIT4);
   UartSetup();                                   // Call function to initialize UART
    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
   sprintf ( (char*)szTemp, "ADuCM320i UART Demo \r\n" ); // Scan string 
   nLen = strlen((char*)szTemp);                  // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	          // loop to send String to UART
       {
          ucTxBufferEmpty = 0;	                  // Clear flag
          UrtTx(pADI_UART,szTemp[i]);             // Load UART Tx register.
          while (ucTxBufferEmpty == 0)            // Wait for UART Tx interrupt
          {
          }
       }
     }
   while (1)
   {
	if (ucButtonPress == 1)
        {
          TestButtonPress();
          ucButtonPress = 0;
        }
   }
}
// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
   // UrtCfg(pADI_UART,B230400,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 230400 baud rate
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                     // Enable Rx, Tx and Rx buffer full Interrupts
   NVIC_EnableIRQ(UART_IRQn);                     // Enable UART interrupt source in NVIC
}
void TestButtonPress(void)
{
   unsigned char i = 0;
   unsigned char nLen = 0;  
  sprintf ( (char*)szTemp, "Button Pressed \r\n" );// Scan string 
   nLen = strlen((char*)szTemp);                  // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	          // loop to send String to UART
       {
          ucTxBufferEmpty = 0;	                  // Clear flag
          UrtTx(pADI_UART,szTemp[i]);             // Load UART Tx register.
          while (ucTxBufferEmpty == 0)            // Wait for UART Tx interrupt
          {
          }
       }
     }
}
void UART_Int_Handler()
{  	
      ucCOMSTA0 = UrtLinSta(pADI_UART);
      ucCOMIID0 = UrtIntSta(pADI_UART);
      if ((ucCOMIID0 & 0x2) == 0x2)	          // Transmit buffer empty
       {
	  ucTxBufferEmpty = 1;
       }	
      if ((ucCOMIID0 & 0x4) == 0x4)	          // Receive byte
       {
	 ucComRx	= UrtRx(pADI_UART);
       }
}
void Ext_Int0_Handler()
{
  
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source
   ucButtonPress = 1;
}



// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




