/**
 *****************************************************************************
   @example  ADC_Sequencer.c
   @brief    ADC Sequencer example file
      Decription: Demo for ADC Sequencer.
      - Sets up ADC Sequencer for all M322 channels 
      - Results converted to Voltage value and string sent to UART.
      - UART configured for 9600-8-N-1
      - 8x VDACs setup as voltage sources also
   @version  V0.4
   @author   ADI
   @date     November 2015
   @par Revision History:
   - V0.1, June 2013: initial version.
   - V0.2, December 2013: Added code for S2 silicon
   - V0.3, January 2014: adjusted bit weight calculation
	 - V0.4, November 2015: Remove unused library references
	 
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


#include <I2cLib.h>
#include <WdtLib.h>
#include <GptLib.h>
#include <UrtLib.h>
#include <DioLib.h>
#include <DacLib.h>
#include <AdcLib.h>

// Function Definitions

void delay(long int);
void ResultsToUart(unsigned long ulVal, unsigned char ucChan);
void ReadSequencer(void);
void UartSetup(void);
void ADCSetUp(void);
void VDACSetup(void);

volatile unsigned char ucSendToUart = 0; 
volatile unsigned char ucTxBufferEmpty = 0;	  // Indicates UART Tx buffer empty
volatile unsigned int uiTest = 0;
unsigned char szTemp[64] = "";		           // Used to store ADC1 result before printing to UART
volatile unsigned char ucCOMSTA0 = 0;          // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;          // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;            // Variable used to read UART Rx buffer contents into 
volatile long ultest,ADCTEST1,ADCTEST2 = 0;

// Variables used to read Sequencer results
volatile long ulAIN0Result = 0;
volatile long ulAIN1Result = 0;
volatile long ulAIN2Result = 0;
volatile long ulAIN3Result = 0;
volatile long ulAIN4Result = 0;
volatile long ulAIN5Result = 0;
volatile long ulAIN6Result = 0;
volatile long ulAIN7Result = 0;
volatile long ulAIN8Result = 0;
volatile long ulAIN9Result = 0;
volatile long ulAIN10Result = 0;
volatile long ulAIN11Result = 0;
volatile long ulAIN12Result = 0;
volatile long ulAIN13Result = 0;
volatile long ulAIN14Result = 0;
volatile long ulAIN15Result = 0;
volatile long ulTempResult = 0;
volatile long ulVrefPResult = 0;
volatile long ulPVDDResult = 0;
volatile long ulIOVddResult = 0;
volatile long ulAVDDResult = 0;
volatile long ulVrefNResult = 0;
volatile unsigned int ucClipLErr = 0;
volatile unsigned int ucClipHErr = 0;
volatile unsigned char ucReadSequence = 0;
float  fVoltage = 0.0;   			              // ADC value converted to voltage
float fVolts = 0.0;
int main (void)
{
   int i,j,k;
   unsigned char nLen = 0; 
   if (pADI_ID->CHIPID == 0x561)                      // check for S2 silicon
   {
      pADI_RESET->RSTKEY =  0x2009;                   //By Default Software and Watchdog resets don't reset the LV die and GPIOs
      pADI_RESET->RSTKEY =  0x0426;                   //During debug it's useful that Software tools have the ability to reset them
      pADI_RESET->RSTCFG = RSTCFG_GPIO_PLA_RETAIN_DIS;//So disable retain function during debug.
   }
   if (pADI_LV->LVID == 0x73)                         // check for S2 silicon
   {
      pADI_LV_RST->LVRST = LVRST_RETAIN_DIS;
   }
                           
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);     // Disable the Watchdog timer resets
   DioOenPin(pADI_GP2,PIN4,1);                  // Configure P2.4 as an output - to toggle LED with Timer 2
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);                   // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   DioSet(pADI_GP2, BIT4);
   pADI_CLKCTL->CLKCON5 = 0x00;                 // Enable clock to all peripherals
   pADI_CLKCTL->CLKCON1 = 0x200;                // PCLK 20MHz, HCLK 80MHz
   UartSetup();
   VDACSetup();

  
   pADI_LV_INT->INTSEL = 0xFE;                  // Enable  LV1 interrupt sources    
   NVIC_EnableIRQ(LVD1_IRQn);			            // Enable LV1 IRQ
   ucReadSequence = 0;
   sprintf ( (char*)szTemp, "ADuCM322 LV Die Sequencer Demo \r\n" ); // Scan string 
   nLen = strlen((char*)szTemp);               // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	        // loop to send String to UART
      {
         ucTxBufferEmpty = 0;	                  // Clear flag
         UrtTx(pADI_UART,szTemp[i]);            // Load UART Tx register.
         while (ucTxBufferEmpty == 0)           // Wait for UART Tx interrupt
         {
         }
      }
   }
   ADCSetUp();                                  // Call function to initialise the ADC 
   AdcGo(ADCCON_C_TYPE_CONT);                   // Turn ADC on - start conversions

   while (1)
   {
       if (ucReadSequence == 1)                 // Sequence complete?
       {
          ReadSequencer();                      // Read off ADC sequencer results and send to UART
          ucReadSequence = 0;                   // Clear flag
          AdcGo(ADCCON_C_TYPE_CONT);           // Re-start conversions
       }
   }
}
void ReadSequencer(void)
{       
   ulAIN0Result = AdcRd(0);                    // Read AIN0 result
   ResultsToUart(ulAIN0Result, 0);             // Send AIN0 result to the UART
   ulAIN1Result = AdcRd(1);                    // Read AIN1 result
   ResultsToUart(ulAIN1Result, 1);             // Send AIN1 result to the UART
   ulAIN2Result = AdcRd(2);                    // Read AIN2 result
   ResultsToUart(ulAIN2Result, 2);             // Send AIN2 result to the UART
   ulAIN3Result = AdcRd(3);                    // Read AIN3 result
   ResultsToUart(ulAIN3Result, 3);             // Send AIN3 result to the UART
   ulAIN4Result = AdcRd(4);                    // Read AIN4 result
   ResultsToUart(ulAIN4Result, 4);             // Send AIN4 result to the UART
   ulAIN5Result = AdcRd(5);                    // Read AIN5 result
   ResultsToUart(ulAIN5Result, 5);             // Send AIN5 result to the UART
   ulAIN6Result = AdcRd(6);                    // Read AIN6 result
   ResultsToUart(ulAIN6Result, 6);             // Send AIN6 result to the UART
   ulAIN7Result = AdcRd(7);                    // Read AIN7 result
   ResultsToUart(ulAIN7Result, 7);             // Send AIN7 result to the UART 
   ulAIN8Result = AdcRd(8);                    // Read AIN8 result
   ResultsToUart(ulAIN8Result, 8);             // Send AIN8 result to the UART
   ulAIN9Result = AdcRd(9);                    // Read AIN9 result
   ResultsToUart(ulAIN9Result, 9);             // Send AIN9 result to the UART
   ulAIN10Result = AdcRd(10);                  // Read AIN10 result
   ResultsToUart(ulAIN10Result, 10);           // Send AIN10 result to the UART
   ulAIN11Result = AdcRd(11);                  // Read AIN11 result
   ResultsToUart(ulAIN11Result, 11);           // Send AIN11 result to the UART
   ulAIN12Result = AdcRd(12);                  // Read AIN12 result
   ResultsToUart(ulAIN12Result, 12);           // Send AIN12 result to the UART
   ulAIN13Result = AdcRd(13);                  // Read AIN13 result
   ResultsToUart(ulAIN13Result, 13);           // Send AIN13 result to the UART
   ulAIN14Result = AdcRd(14);                  // Read AIN14 result
   ResultsToUart(ulAIN14Result, 14);           // Send AIN14 result to the UART
   ulAIN15Result = AdcRd(15);                  // Read AIN15 result
   ResultsToUart(ulAIN15Result, 15);           // Send AIN15 result to the UART
   ulVrefPResult = AdcRd(23);                  // Read VREF result
   ResultsToUart(ulVrefPResult, 23);           // Send VREF result to the UART
}
// Function used to initalise UART
void ResultsToUart(unsigned long ulVal, unsigned char ucChan)
{
   unsigned char i = 0;
   unsigned char nLen = 0; 

   if (pADI_LV->LVID == 0x71)                     // check for S1 silicon
   {
      fVolts   = (2.5 / 65535);                   // Internal reference, calculate lsb size in volts
      fVoltage = ((ulVal>>12) * fVolts);          // Calculate ADC result in volts
      fVoltage = (fVoltage *2.51)/2.5;
   }
   else if (pADI_LV->LVID == 0x73)                // check for S2 silicon
   {
      fVolts   = (2.51 / 65535);                 // Internal reference, calculate lsb size in volts
      fVoltage = ((ulVal>>12) * fVolts);          // Calculate ADC result in volts
   }
   else
   {
      fVolts   = (2.5 / 65535);                   // Internal reference, calculate lsb size in volts
      fVoltage = ((ulVal>>12) * fVolts);          // Calculate ADC result in volts
   }
   sprintf ( (char*)szTemp, "%fV,%d \r\n",fVoltage,ucChan );// Scan string with the ADC1 Result  
   nLen = strlen((char*)szTemp);               // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	        // loop to send ADC result	to UART
       {
          ucTxBufferEmpty = 0;	              // Clear flag
          UrtTx(pADI_UART,szTemp[i]);          // Load UART Tx register.
          while (ucTxBufferEmpty == 0)         // Wait for UART Tx interrupt
          {
          }
       }
    }
}
// Function used to initalise VDACs and set output voltage
void VDACSetup(void)
{
    DacCfg(pADI_VDAC0,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC0
    DacWr(pADI_VDAC0,0x08000000);                           // Set VDAC0 to mid-scale
    DacCfg(pADI_VDAC1,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC1
    DacWr(pADI_VDAC1,0x04000000);                           // Set VDAC1 to quarter-scale
    DacCfg(pADI_VDAC2,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC2
    DacWr(pADI_VDAC2,0x0C000000);                           // Set VDAC2 to 3/4-scale
    DacCfg(pADI_VDAC3,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC2
    DacWr(pADI_VDAC3,0x08000000);                           // Set VDAC2 to mid-scale  
    DacCfg(pADI_VDAC4,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC4
    DacWr(pADI_VDAC4,0x08000000);                           // Set VDAC4 to mid-scale
    DacCfg(pADI_VDAC5,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC5
    DacWr(pADI_VDAC5,0x04000000);                           // Set VDAC5 to quarter-scale
    DacCfg(pADI_VDAC6,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC6
    DacWr(pADI_VDAC6,0x0C000000);                           // Set VDAC6 to 3/4-scale
    DacCfg(pADI_VDAC7,DACCON_MODE_12BIT,INT_REF);           // Turn on VDAC7
    DacWr(pADI_VDAC7,0x08000000);                           // Set VDAC7 to mid-scale 

}

// Function used to initalise ADC
void ADCSetUp(void)
{
   if (pADI_LV->LVID == 0x71)                     // check for S1 silicon
   {
      AdcSpeed(160);                                   // Value for 100KSPS
   }
   else if (pADI_LV->LVID == 0x73)                // check for S2 silicon
   {
      AdcSpeed(200);                                   // Value for 100KSPS
   }
 
   AdcGo(ADCCON_C_TYPE_NO);                    // Power up ADC and set to idle mode
  // AdcBuf(IBUFCON_IBUF_PD_NONE,
      //IBUFCON_IBUF_BYP_BOTH,ADCCON_REFB_PUP);
   AdcSeqCfg(ADCSEQ_EN,AIN0_Seq|AIN1_Seq|AIN2_Seq|
   AIN3_Seq|AIN4_Seq|AIN5_Seq|AIN6_Seq|AIN7_Seq|
   AIN8_Seq|AIN9_Seq|AIN10_Seq|AIN11_Seq|AIN12_Seq|
   AIN13_Seq|AIN14_Seq|AIN15_Seq|VREFP_Seq,0x10);

   AdcSeqDiffCfg(0x11,0x11,0x11,0x11);
   AdcSeqStart(ADCSEQ_ST);

  // pADI_ADC->ADCSEQC = 0x0808C631;             // All single ended measurements
   pADI_ADC->ADCCMP = 0x10001;                 // Enable digital comparator on V<mid scale
 //  pADI_ADC->ADCCMP = 0x10003;               // Enable digital comparator on V>mid scale
}

// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                       // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
//   UrtCfg(pADI_UART,B115200,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 115200 baud rate
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                  // Enable Rx, Tx and Rx buffer full Interrupts
   NVIC_EnableIRQ(UART_IRQn);                  // Enable UART interrupt source in NVIC
}
// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}
void LV0_Int_Handler()
{
    ADCTEST1 = pADI_ADC->ADCDAT[1];
    pADI_LV_INT->INTCLR = 0x1;                 // clear Irq source
    pADI_GP2->GPTGL = 0x2;                     // Toggle P2.1
}
//ADC IRQ Handler function
void LV1_Int_Handler()
{
   unsigned char ucLVIrqStatus = 0;
   
   ucLVIrqStatus = pADI_LV_INT->INTSTA;
   if ((ucLVIrqStatus & 0x1) == 0x1)           // ADC software conversion IRQ
   {
       pADI_LV_INT->INTCLR = 0x1;              // clear Irq source
      ADCTEST1 = pADI_ADC->ADCDAT[4];
      pADI_GP2->GPTGL = 0x10;                  // Toggle P2.4
      uiTest++;
      if (uiTest == 256)
      {
         ucSendToUart = 1;
         uiTest = 0;
         AdcGo(ADCCON_C_TYPE_NO);              // Disable ADC conversions until sequence results sent to UART
      }
      
   }
   if ((ucLVIrqStatus & 0x2) == 0x2)           // ADC Sequencer IRQ
   {
      uiTest++;
      pADI_LV_INT->INTCLR = 0x2;               // clear Irq source
      pADI_ADC->ADCCON = 0x280;
      ucReadSequence = 1;
   }
   if ((ucLVIrqStatus & 0x4) == 0x4)           // ADC Digital Comparator IRQ
   {
      uiTest++;
      pADI_LV_INT->INTCLR = 0x4;               // clear Irq source
   }
   if ((ucLVIrqStatus & 0x10) == 0x10)         
   {
      uiTest++;
      pADI_LV_INT->INTCLR = 0x10;              // clear Irq source
   }
   if ((ucLVIrqStatus & 0x20) == 0x20)         
   {
      uiTest++;
      pADI_LV_INT->INTCLR = 0x20;              // clear Irq source
   }
   if ((ucLVIrqStatus & 0x40) == 0x40)         // D2D read ECC error IRQ
   {
      uiTest++;
      pADI_LV_INT->INTCLR = 0x40;              // clear Irq source
   }
   if ((ucLVIrqStatus & 0x80) == 0x80)         // D2D Write ECC error IRQ
   {
      uiTest++;
      pADI_LV_INT->INTCLR = 0x80;              // clear Irq source
   }
}

//UART IRQ Handler function
void UART_Int_Handler()
{  	
   ucCOMSTA0 = UrtLinSta(pADI_UART);
   ucCOMIID0 = UrtIntSta(pADI_UART);
   if ((ucCOMIID0 & 0x2) == 0x2)	          // Transmit buffer empty
   {
	  ucTxBufferEmpty = 1;
   }	
   if ((ucCOMIID0 & 0x4) == 0x4)	          // Receive byte
   {
	  ucComRx	= UrtRx(pADI_UART);
   }
}
