/**
 *****************************************************************************
   @example  Cache.c
   @brief    Example of different cacheing options.
      Decription:
      This example switches between different cacheing options and executes
      a short test for each option.  P1.6 is pulsed before each test and 
      pulsed twice after each test so that the execution time can be 
      visualised on an oscilloscope.
      This file requires the L1 cache to be enabled as for instance in the 
      file startup_ADuCM320L1.s.  In addition, in the tools setup, the file 
      L1Ram.s must be assigned to the L1 cache address range 
      0x10000000-0x10004000.
   
   @version    V0.3
   @author     ADI
   @date       December 2015
   @par Revision History:
      - V0.1, July 2014: Initial version.
      - V0.2, July 2014: Use modified cache functions.
      - V0.3, December 2015: ADuCMxxx.h changed to DeviceHeader.h

All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.  It is the 
responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <DeviceHeader.h>
#include <FeeLib.h>
#include <DioLib.h>

int DioPulse(ADI_GPIO_TypeDef *pGP,int iBit,int iCnt);
extern int CacheSpd(void);
extern int RamSpd(void);

int main(void)
	{
	static int	    i1;
                                                          
	pADI_WDT->T3CON = 0xc9;         //Disable watchdog.
	pADI_GP0->GPOE = 0x07;
   for(i1=0; i1<1000000; i1++)    DioPulse(pADI_GP0,2,4);
   
	while(1)
		{
      FeeCacheEn(CACHESETUP_IEN_EN);
      FeeCacheClr(CACHESETUP_IINIT_EN);
      DioPulse(pADI_GP0,0,1);
      DioPulse(pADI_GP0,2,1);
      CacheSpd();
      CacheSpd();
      FeeCacheEn(0);
      CacheSpd();
      CacheSpd();
      RamSpd();
      RamSpd();
		}
	}

/*Pulse port pGP bit iBit iCnt times.  */
int DioPulse(ADI_GPIO_TypeDef *pGP, int iBit, int iCnt)
   {
   int i1;

   for(i1=0; i1<iCnt; i1++)
      {
      if(pGP==pADI_GP1)   pADI_GPIO_FIX->GP1SET = 1<<iBit;
      else    pGP->GPSET = 1<<iBit;
      if(pGP==pADI_GP2)   pADI_GPIO_FIX->GP2CLR = 1<<iBit;
      else    pGP->GPCLR = 1<<iBit;
      }
   return 1;
   }
    
