/**
 *****************************************************************************
   @example  DIO.c
   @brief    Demonstrate the use of the digital IO functions.
             In the while loop the LED (2.4) is toggled
             
   @version  V1.0
   @author   ADI
   @date     November 2015 

              
All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>

#include <WdtLib.h>
#include <DioLib.h>

unsigned char ucPort = 0;

void delay(long int);

int main (void)
{
   int i,j,k;
   WdtGo(T3CON_ENABLE_DIS);
   
	DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   DioOen(pADI_GP0,0x3);            // Pin 0.0 and 0.1 as outputs
   DioSet(pADI_GP0,0x3);            // Set 0.0 and 0.1 HIGH
   delay (1000000);
   DioClr(pADI_GP0,0x3);            // Clear 0.0 and 0.1
   
   delay(1000000);
   DioOen(pADI_GP0,0x0);            // Disable output path for port 0
   
   DioPul(pADI_GP0,0xFF);           // Enable pull-ups
   delay (1000000);
   ucPort = DioRd(pADI_GP0);        // Read port 0 inputs
	
   while (1)
   {
	   DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      delay(1000000);
      delay(1000000);
      delay(1000000);
      delay(1000000);
	}
}

// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




