/**
 *****************************************************************************
   @example  FlashDma.c
   @brief    Flash DMA example
      This projects shows how to use the DMA to store a large array of data
      from RAM into flash

   @version  V0.2
   @author   ADI
   @date     December 2015
	 @par Revision History:
   - V0.1, June 2013: initial version.
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h
	 
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.


**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


//#include <ClkLib.h>
#include <IntLib.h>
#include <WdtLib.h>
#include <DioLib.h>
#include <DmaLib.h>


#define DATA_SZ (64)
void delay(long int);
void SPI1INIT(void);
void DMAINIT(void);


unsigned int done_cnt = 0;
int dma_done = 0;

unsigned char ucIrqCnt = 0;                        // Used to count number of SPI1 DMA interrupts
unsigned char ucWriteData = 0;                      // Flag used to setup 2nd and subsequent transfers
unsigned int uiFlashData[DATA_SZ];
unsigned long ulFEESTA;
unsigned int uiAdr = 0x21000;

int main (void)
{
   unsigned int numDmaTran = 0;
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);                                   // Disable the Watchdog timer resets
   
   DioOenPin(pADI_GP2,PIN4,1);                                                // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   for(i=0; i<DATA_SZ; i++)
   {
      uiFlashData[i] = 0x12345600+i;
   }
   
   DMAINIT();                                                                 // Initialize DMA controller
   
   EiCfg(EXTINT0,INT_EN,INT_RISE);                                            // Enable EINT0 - used to triger transfer
   NVIC_EnableIRQ(EINT0_IRQn);                                                // Enable IRQ0 interrupt
   NVIC_EnableIRQ(DMA_FLASH_IRQn);                                            // Flash DMA interrupt enable
   
   DmaPeripheralStructSetup(Flash_C,DMA_DSTINC_NO|
      DMA_SRCINC_WORD|DMA_SIZE_WORD);                                   // Setup Flash DMA structure  
   
   numDmaTran = DATA_SZ;
   DmaStructPtrOutSetup(Flash_C,numDmaTran,(unsigned char*)uiFlashData);// Setup source/destination pointer fields
   DmaSet(0,Flash_B,0,0);                                               // Enable DMA structure for flash
   DmaGo(Flash_C,numDmaTran,DMA_BASIC);                                 // Enable Flash DMA structure
   
   pADI_FEE->FEEFLADR = uiAdr;
   pADI_FEE->FEEKEY = 0xF123F456;
   pADI_FEE->FEECON1 |= (FEECON1_KHDMA_EN);                           // Enable Flash DMA mode, this starts the DMA transfer
   
   while (1)
   {
      delay(0x60000);         // Delay routine
      if ( (dma_done ==1) & (ucWriteData == 1) & (uiAdr<0x30000))
      {
         dma_done = 0;
         ucWriteData = 0;
         uiAdr += (DATA_SZ*4);
         DmaSet(0,Flash_B,0,0);                                        // Enable DMA structure for flash
         DmaGo(Flash_C,numDmaTran,DMA_BASIC);                          // Enable Flash DMA structure
         
         pADI_FEE->FEEFLADR = uiAdr;
         pADI_FEE->FEEKEY = 0xF123F456;
         pADI_FEE->FEECON1 |= (FEECON1_KHDMA_EN);
      }
   }
}


void DMAINIT(void)  
{
   DmaBase();                 
}
// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}
void WakeUp_Int_Handler(void)
{
  
}
void Ext_Int2_Handler ()
{           

}
void Ext_Int0_Handler ()
{         
   EiClr(EXTINT0);                                          // Clear EINT0 interrupt flag
   ucWriteData = 1;
}

void UART_Int_Handler ()
{

} 
void DMA_Flsh_Int_Handler()
{
   pADI_FEE->FEEKEY = 0xF123F456;
   pADI_FEE->FEECON1 &= (~FEECON1_KHDMA_EN);           // Disable Flash DMA mode
   done_cnt ++;
   dma_done = 1;
}



