/** 
     @example  I2CMasterDma.c
     @brief I2C Master dma democode. 
     Decription: Sends 16 bytes to the Slave when External IRQ0 (P0.3) is asserted
      - Interfaces to slave device via I2C0
      Can receive 16 bytes from Slave when XIRQ1 (P1.7) is asserted - will send these to UART
   @version  V0.2
   @author   ADI
   @date     December 2015 
	 @par Revision History:
	 - V0.1, May 2014: initial version.
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h 

 *****************************************************************************

              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <DeviceHeader.h>

#include <I2cLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <DmaLib.h>


// Function Definitions
void UartSetup(void);
void I2CMasterSetup(void);
void delay(long int);

unsigned int ucBytesToRD = 0;			              // Stores number of bytes to be read from Slave via I2C
unsigned char rxDat[256];	                       // 256 byte buffer-array for I2C receive data
volatile unsigned char ucSendI2CPacket = 0;	     // Used in main loop to indicate I2C message is ready for transmission
volatile unsigned char ucMsgType = 0;             // Indicates I2C read or write
volatile unsigned int ucLength = 0;               // Indicates number of bytes for I2C read or write
volatile unsigned int uiTest = 0;
unsigned char ucTransfer[512];	                 // Buffer used during I2C "writes to Slave"	
unsigned char ucDmaDone = 1;
unsigned char ucTComp = 1;

int main (void)
{
   int i,j,k = 0;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer resets
   DioPul(pADI_GP0, 0xFF);
   DioPul(pADI_GP1, 0xFF);
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   DmaBase(); 
   I2CMasterSetup();                              // Call function to initialize I2C master

    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 - P0.3 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
   EiCfg(EXTINT1, INT_EN, INT_FALL);              // Enable External IRQ 1 - P1.7 
   NVIC_EnableIRQ(EINT1_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
	DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);
   
   ucLength = 0x10;
   for (i = 0; i< ucLength; i++)
   {
      ucTransfer[i] = i;
   }

   while (1)
   {
   if (ucSendI2CPacket == 1 & ucTComp == 1)                      // request received via XIRQ to send I2C Packet
		{
         ucSendI2CPacket = 0;
			if (ucMsgType == 0)                                // Write packet
			{
            for (i = 0; i< ucLength; i++)
            {
               ucTransfer[i] += 0x10;
            }
            I2cFifoFlush(pADI_I2C0,MASTER,ENABLE);	         // Flush Master FIFO
            I2cFifoFlush(pADI_I2C0,MASTER,DISABLE);
            ucDmaDone = 0;
            ucTComp = 0;
			 	DmaPeripheralStructSetup(I2C0M_C,DMA_DSTINC_NO|
               DMA_SRCINC_BYTE|DMA_SIZE_BYTE);     
            DmaSet(0,I2C0M_B,0,0);
            DmaStructPtrOutSetup(I2C0M_C,ucLength,ucTransfer);          // Setup source/destination pointer fields
            DmaGo(I2C0M_C,ucLength,DMA_BASIC);                          // Enable I2C0 Master DMA structure
            I2cMWrCfg(pADI_I2C0,0xA0);                                  // set i2c address	 (LSB = 0, Master Write)
			}
			else                                                // read packet
			{
            I2cFifoFlush(pADI_I2C0,MASTER,ENABLE);	         // Flush Master FIFO
            I2cFifoFlush(pADI_I2C0,MASTER,DISABLE);
            ucDmaDone = 0;
            ucTComp = 0;
				DmaPeripheralStructSetup(I2C0M_C,DMA_DSTINC_BYTE|
               DMA_SRCINC_NO|DMA_SIZE_BYTE);     
            DmaSet(0,I2C0M_B,0,0);
            DmaStructPtrInSetup(I2C0M_C,ucLength,rxDat);                // Setup source/destination pointer fields
            DmaGo(I2C0M_C,ucLength,DMA_BASIC);                          // Enable I2C0 Master DMA structure
            I2cMRdCfg(pADI_I2C0,0xA1, ucLength, DISABLE);               // set i2c address	 (LSB = 1, Master read)	
			}
		}
      delay(10000);
	}
}
// Function used to setup the I2C0 as master
void I2CMasterSetup(void)
{
   DioCfg(pADI_GP0,0x500);                         // Configure P0[4:7] as I2C pins

 //  I2cBaud(pADI_I2C0,0x62,0x63);                 // Configure baud rate for 100khz
   I2cBaud(pADI_I2C0,0x13,0x15);                   // 400kHz Baud rate
   I2cMCfg(pADI_I2C0,I2CMCON_MTXDMA|I2CMCON_MRXDMA,
   I2CMCON_IENCMP_EN|
   I2CMCON_IENACK_EN|
   I2CMCON_IENALOST_EN
//    I2CMCON_IENMTX_EN|
//    I2CMCON_IENMRX_EN,
   ,I2CMCON_MASEN_EN);                             // Enable I2C0 master mode. Enable Interrupt sources
   NVIC_EnableIRQ(I2C0M_IRQn);                     // Enable i2C0 Master interrupt source
   NVIC_EnableIRQ(DMA_I2C0M_IRQn);
   
   DmaPeripheralStructSetup(I2C0M_C,DMA_DSTINC_NO|
      DMA_SRCINC_BYTE|DMA_SIZE_BYTE);                        

}

void Ext_Int0_Handler()
{
   ucSendI2CPacket = 1;
   ucMsgType = 0;
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source

}
void Ext_Int1_Handler()
{
   ucSendI2CPacket = 1;
   ucMsgType = 1;
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x2;                   // Clear the interrupt1 source

}
void I2C0_Master_Int_Handler()
{
   unsigned int I2CMSTATUS = 0;
   
   I2CMSTATUS = I2cSta(pADI_I2C0,MASTER);

   if((I2CMSTATUS & 0x200) == 0x200)			     // Rx overflow
	{
		uiTest++;								
	}
	if((I2CMSTATUS & 0x100) == 0x100)			     // Transmission Complete/Stop detected
	{
		ucTComp = 1;	
	}
	if((I2CMSTATUS & 0x80) == 0x80)			        // NACK Data detected
	{
		uiTest++;								
	}	
   if((I2CMSTATUS & 0x40) == 0x40)			        // NACK data detected
	{
		uiTest++;							
   }
	if((I2CMSTATUS & 0x10) == 0x10)			        // NACK Address detected
	{
		uiTest++;							
   }
}
void I2C1_Master_Int_Handler()
{
   
}

void DMA_I2C0_M_Int_Handler()
{
   ucDmaDone = 1;
   //DioTgl(pADI_GP2, BIT4);
}


// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




