/**
 *****************************************************************************
   @example  MdioInt.c
   @brief    Example code to enable, receive and handle MDIO interrupts.
             
   @version  V0.4
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, June 2013: initial version.
   - V0.2, January 2014: Adapted for S2 Silicon
   - V0.3, Fixed whitespace.
	 - V0.4, December 2015: Changed ADuCM3xx.h to DeviceHeader.h
    
              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include "DeviceHeader.h"
#include "MdLib.h"
#include "WdtLib.h"
#include "DioLib.h"
#include "UrtLib.h"

int DioPulse(ADI_GPIO_TypeDef *pGP,int iBit,int iCnt);

int iMdioSta;

void MDIO_Int_Handler()
   {
   DioPulse(pADI_GP0,1,1);
   iMdioSta = MdSta();	
   if(iMdioSta&1)   
      {
      DioPulse(pADI_GP0,2,2);
      iMdioSta += MdRxd()<<16;
      }
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&2)
      {
      DioPulse(pADI_GP0,2,2);
      iMdioSta += MdRxd()<<16;
      }
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&4)
      {
      DioPulse(pADI_GP0,2,2);
      MdTxd(iMdioSta+0x1d00);
      }
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&8)
      {
      DioPulse(pADI_GP0,2,2);
      MdTxd(iMdioSta+0x2d00);
      }
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&0x10)   DioPulse(pADI_GP0,2,2);
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&0x20)   DioPulse(pADI_GP0,2,2);
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&0x40)   DioPulse(pADI_GP0,2,2);
   else   DioPulse(pADI_GP0,2,1);    
   if(iMdioSta&0x80)   DioPulse(pADI_GP0,2,2);
   else   DioPulse(pADI_GP0,2,1);    
   DioPulse(pADI_GP0,1,1);
   }

int main(void)
   {
   int i1;
   char    pc1[100];

   pADI_GP0->GPOE = 0x07;
   for(i1=0; i1<2000000; i1++)    DioPulse(pADI_GP0,2,1);
   WdtGo(T3CON_ENABLE_DIS);
   DioPulse(pADI_GP0,2,1);
   DioPulse(pADI_GP0,2,1);

   pADI_GP3->GPCON = 0x0555;   //Put MDIO on P3. 
   MdInit(0,0,1);              //MDIO 5 bit with pushpull.
   MdCfg(5,0x1f,1);            //PHYADR = 5, DEVADR = 1. 
   MdTxd(0x55AA);
   MdIen(MDIEN_MD_WRFI|MDIEN_MD_ADRI|MDIEN_MD_INCFI|MDIEN_MD_RDFI); 
   NVIC_EnableIRQ(MDIO_IRQn);  //Enable MDIO interrupt source in NVIC.
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
   while(1)
      {
//		DioPulse(pADI_GP0,1,5);
      if(iMdioSta)
         {
         sprintf(pc1,"%08x\n\r",iMdioSta);
         i1 = 0;
         while(pc1[i1++]>0 && i1<15)
            {
            while(!(UrtLinSta(pADI_UART)&COMLSR_THRE));
            UrtTx(pADI_UART,pc1[i1-1]);
            }
         iMdioSta = 0;
         }
      }
   }

int DioPulse(ADI_GPIO_TypeDef *pGP,int iBit,int iCnt)
   {
   int i1;

   for(i1=0; i1<iCnt; i1++)
      {
      if(pGP==pADI_GP1)   pADI_GPIO_FIX->GP1SET = 1<<iBit;
      else    pGP->GPSET = 1<<iBit;
      if(pGP==pADI_GP2)   pADI_GPIO_FIX->GP2CLR = 1<<iBit;
      else    pGP->GPCLR = 1<<iBit;
      }
   return 1;
   }
