/** 
     @example  SPIMaster.c
     @brief SPI Master Basic democode. 
     Description: Sends 16x bytes to the Slave when External IRQ0 is asserted
      - Interfaces to slave device via SPI1
   @version  V0.5
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, March 2013: Initial release
   - V0.2, April 2013: Added delay before program starts
   - V0.3, January 2014: Fixed baud rate parameter, fixed interrupt logic.
   - V0.4, March 2014: Disable SPI HFM mode
	 - V0.5, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

 *****************************************************************************


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>

#include <SpiLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>



// Function Definitions
void SPI1MasterSetup(void);
unsigned char SPI1WritePacket(unsigned int length);// Writes 5x Bytes to Slave
void delay(long int);


volatile unsigned char ucRdLength = 0;	           // Stores number of bytes to be read from Slave via SPI1
volatile unsigned char StartByteGone = 0;
unsigned int ucBytesToRD = 0;			              // Stores number of bytes to be read from Slave via SPI1
volatile unsigned int rxDat[256];	              // 256 byte buffer-array for SPI1 receive data
volatile unsigned char ucRxCount = 0;             // rxDat array control variable.
volatile unsigned char ucSPIWrCnt = 0;            // SPI1 Tx counter
volatile unsigned char ucSendSPIPacket = 0;	     // Used in main loop to indicate SPI message is ready for transmission
volatile unsigned char ucReceiveInProgress =0;    // Indicates SPI Rx transfer in progress
volatile unsigned char ucTransmitInProgress =0;   // Indicates SPI Tx transfer in progress
volatile unsigned char ucMsgType = 0;             // Indicates SPI read or write
volatile unsigned int ucLength = 0;               // Indicates number of bytes for SPI read or write
volatile unsigned char ucLoadCnt = 0;
volatile unsigned char ucTransfer[128];	        // Buffer used during SPI "writes to Slave"	



int main (void)
{
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer resets
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   DioPul(pADI_GP0, 0xFF);
   DioPul(pADI_GP1, 0xFF);
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   SPI1MasterSetup();                             // Call function to initialize SPI1 master
    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 - P0.3 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
 
	DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);

   while (1)
   {
   if (ucSendSPIPacket == 1)                      // request received  to send SPI Packet
		{
			if (ucMsgType == 0)                      // Write packet?
			{
			 	ucLength = 16;
            ucBytesToRD = 5;
            ucTransfer[0] = 0x11;
            ucTransfer[1] = 0x22;
            ucTransfer[2] = 0x33;
            ucTransfer[3] = 0x44;
            ucTransfer[4] = 0x55;
            ucTransfer[5] = 0x00;
            ucTransfer[6] = 0x66;
            ucTransfer[7] = 0x77;
            ucTransfer[8] = 0x88;
            ucTransfer[9] = 0x99;
            ucTransfer[10] = 0xaa;
            ucTransfer[11] = 0xbb;
            ucTransfer[12] = 0xcc;
            ucTransfer[13] = 0xdd;
            ucTransfer[14] = 0xee;
            ucTransfer[15] = 0xff;
            SPI1WritePacket(ucLength);
				ucSendSPIPacket = 0;

			}
		}	
	}
}
// SPI1MasterSetup used to setup the SPI1 as master
void SPI1MasterSetup(void)
{
   DioCfg(pADI_GP1,0xAA00);                      // Configure P1[4:7] as SPI1 pins     
   SpiBaud(pADI_SPI1,63,SPIDIV_BCRST_EN|          // Select 625khz SPI CLock, Enable chip select edge detect IRQ source, 
   SPIDIV_CSIRQ_EN);                              // Enable resetting of internal bit-shift counter
   SpiCfg(pADI_SPI1,SPICON_MOD_TX1RX1,            // Enable interrupt after 1 byte transfer
   SPICON_MASEN_EN,SPICON_CON_EN|SPICON_RXOF_EN|  // Enable master, SPI mode, Rx overflow interrupt
   SPICON_ZEN_EN|SPICON_TIM_TXWR|                 // transfer 0 if FIFO is empty, initiate trasnfer on write to SPITX register
   SPICON_CPOL_HIGH|SPICON_CPHA_SAMPLETRAILING|   // SPI clock idles high, samples on falling clock edge
   SPICON_ENABLE_EN);                             // Enable SPI1
   NVIC_EnableIRQ(SPI1_IRQn);                     // Enable SPI1 interrupt source
}

// This function Starts an SPI1 Write sequence
unsigned char SPI1WritePacket(unsigned int length )
{
	// Transmit
	ucTransmitInProgress = 1;                      // Number of bytes to transfer
   SpiCfg(pADI_SPI1,SPICON_MOD_TX1RX1,            // Enable interrupt after 1 byte transfer
   SPICON_MASEN_EN,SPICON_CON_EN|SPICON_RXOF_EN|  // Enable master, SPI mode, Rx overflow interrupt
   SPICON_ZEN_EN|SPICON_TIM_TXWR|                 // transfer 0 if FIFO is empty, initiate trasnfer on write to SPITX register
   SPICON_CPOL_HIGH|SPICON_CPHA_SAMPLETRAILING|   // SPI clock idles high, samples on falling clock edge
   SPICON_ENABLE_EN);                             // Enable SPI1

	SpiFifoFlush(pADI_SPI1,SPICON_TFLUSH_EN,
                     SPICON_RFLUSH_EN);	        // Flush Master Tx FIFO
	SpiFifoFlush(pADI_SPI1,SPICON_TFLUSH_DIS,
      SPICON_RFLUSH_DIS);
   ucSPIWrCnt = 0;
   SpiTx(pADI_SPI1,ucTransfer[ucSPIWrCnt++]);    // send SPI1 byte    	
	
	while (ucTransmitInProgress == 1)
	{}
	
	return 1;
}


void Ext_Int0_Handler()
{
   ucSendSPIPacket = 1;
   ucMsgType = 0;
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source

}

void SPI1_Int_Handler()
{
   unsigned int SPI1MSTATUS = 0;
   
   SPI1MSTATUS = SpiSta(pADI_SPI1);               // Read SPI1STA register
    // Transmit
	if((SPI1MSTATUS & 0x20) == 0x20)  	           // Master Transmit IRQ	
	{											    
		if (ucSPIWrCnt < ucLength)
      {                                           // Tx FIFO ready for another byte
        SpiTx(pADI_SPI1,ucTransfer[ucSPIWrCnt++]);// send SPI1 byte  
        rxDat[ucRxCount++] = SpiRx(pADI_SPI1);     // read SPI1 FIFO
      }
		else 
		{
			ucTransmitInProgress = 0;
		}
   }			
	// Receive
	if((SPI1MSTATUS & 0x40) == 0x40)				        // Master Recieve IRQ
	{
		if (ucReceiveInProgress == 1)
          rxDat[ucRxCount++] = SpiRx(pADI_SPI1);     // read SPI1 FIFO
		if(ucRxCount == (ucBytesToRD))
			ucReceiveInProgress = 0;	
   }
  if((SPI1MSTATUS & 0x4000) == 0x4000)				  // Rising edge detected on SPI Chip select pin
	{		
      if (ucTransmitInProgress == 0)
      {
         ucRxCount = 0;
         ucSPIWrCnt = 0;
          SpiFifoFlush(pADI_SPI1,SPICON_TFLUSH_DIS,
                     SPICON_RFLUSH_EN);	        // Flush Master Rx FIFO
	      SpiFifoFlush(pADI_SPI1,SPICON_TFLUSH_DIS,
           SPICON_RFLUSH_DIS);
      }
   }    
}

// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




