/** 
     @example  SPISlave.c
     @brief SPI Slave Basic demo code. 
     Decription: Sends/receives data
   @version  V0.2
   @author   ADI
   @date     December 2015
	 @par Revision History:
	 - V0.1, April 2013: initial version.
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

 *****************************************************************************


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>

#include <SpiLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>



// Function Definitions
void SPI1SlaveSetup(void);
void delay(long int);


volatile unsigned char ucRdLength = 0;	           // Stores number of bytes to be read from Slave via SPI1
volatile unsigned char StartByteGone = 0;
unsigned int ucBytesToRD = 0;			              // Stores number of bytes to be read from Slave via SPI1
volatile unsigned int rxDat[256];	              // 256 byte buffer-array for SPI1 receive data
volatile unsigned char ucRxCount = 0;             // rxDat array control variable.
volatile unsigned char ucSPIWrCnt = 0;            // SPI1 Tx counter
volatile unsigned char ucSendSPIPacket = 0;	     // Used in main loop to indicate SPI message is ready for transmission
volatile unsigned char ucReceiveInProgress =0;    // Indicates SPI Rx transfer in progress
volatile unsigned char ucTransmitInProgress =0;   // Indicates SPI Tx transfer in progress
volatile unsigned char ucMsgType = 0;             // Indicates SPI read or write
volatile unsigned int ucLength = 0;               // Indicates number of bytes for SPI read or write
volatile unsigned char ucLoadCnt = 0;
volatile unsigned char ucTransfer[] ={0xA, 0xB,0xC,// Buffer used during SPI "writes to Slave"	
  0x10,0x11,0x12,0x13,0x14,0x14,0x15,0x16,0x17,
  0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F};


int main (void)
{
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);       // Disable the Watchdog timer resets
   	DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   SPI1SlaveSetup();                              // Call function to initialize SPI1 Slave

 	DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);

   while (1)
   {
      pADI_GP2->GPTGL = 0x10;                     // Toggle P2.4
      delay(0x8000);
	}
}
// SPI1SlaveSetup used to setup the SPI1 as master
void SPI1SlaveSetup(void)
{
    DioCfg(pADI_GP1,0xAA00);                      // Configure P1[4:7] as SPI1 pins     
   SpiCfg(pADI_SPI1,SPICON_MOD_TX1RX1,            // Enable interrupt after 1 byte transfer
   SPICON_MASEN_DIS,SPICON_CON_EN|SPICON_OEN|     // Enable Slave, SPI mode, Continuous transfers, MISO as normal 
   SPICON_RXOF_EN|SPICON_CPOL_HIGH|               // Rx overflow interrupt, SPI clock idles high, 
   SPICON_CPHA_SAMPLETRAILING|SPICON_ENABLE_EN);  //  samples on falling clock edge
   
   pADI_SPI1->SPIDIV |=0x140;                     // Enable an interrupt when CS goes high
   NVIC_EnableIRQ(SPI1_IRQn);                     // Enable SPI1 interrupt source
   SpiTx(pADI_SPI1,ucTransfer[0]);                // Init SPI1 Tx FIFO
   ucSPIWrCnt = 1;
   ucBytesToRD = 5;
   ucReceiveInProgress = 1;
}


void SPI1_Int_Handler()
{
   unsigned int SPI1MSTATUS = 0;
   
   SPI1MSTATUS = SpiSta(pADI_SPI1);               // Read SPI1STA register
    // Transmit
	if((SPI1MSTATUS & 0x20) == 0x20)  	           // Master Transmit IRQ	
	{											    
	
   }			
	// Receive
	if((SPI1MSTATUS & 0x40) == 0x40)				        // Master Recieve IRQ
	{
		rxDat[ucRxCount++] = SpiRx(pADI_SPI1);         // read SPI1 FIFO
      SpiTx(pADI_SPI1,ucTransfer[ucSPIWrCnt++]);     // send SPI1 byte 
		if(ucRxCount == (ucBytesToRD))
			ucReceiveInProgress = 0;	
   } 
   if((SPI1MSTATUS & 0x4000) == 0x4000)				  // Rising edge detected on SPI Chip select pin
	{
		if (ucReceiveInProgress == 0)
      {
          ucRxCount = 0;
          ucSPIWrCnt = 0;
         ucReceiveInProgress = 1;
         SpiTx(pADI_SPI1,ucTransfer[0]);             // Preload SPI Tx buffer for next transfer 
      }
   } 
}

// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




