/** @example UartExample.c
   @brief   Sends string to PC at 9600 baud
    LED toggles and string sent to DMA UART for every IRQ0 (push button IRQ)

   @version V0.2
   @author  ADI
   @date     December 2015
	 @par Revision History:
	 - V0.1, April 2013: initial version
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h
	 
 *****************************************************************************


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


#include <WdtLib.h>
#include <GptLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <UrtLib.h>
#include <DmaLib.h>

void DMAINIT(void);
void UartSetup(void);                             // UART Setup function
void TestButtonPress(void);                       // Sends string to UART when IRQ0 asserted
volatile unsigned char ucCOMSTA0 = 0;             // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;             // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;               // Variable used to read UART Rx buffer contents into 
unsigned char ucTxBufferEmpty  = 0;		  // Used to indicate that the UART Tx buffer is empty
unsigned char szTemp[64] = "";		          // Used to store ADC1 result before printing to UART
unsigned char ucButtonPress = 0;                  // used to detect IRQ0 interrupt

unsigned char ucIrqCnt1 = 0;   										// Used to count number of UART DMA interrupts
unsigned char ucIrqCnt2 = 0;   										// Used to count number of UART DMA interrupts

int main (void)
{
   int i,j,k = 0;
   unsigned char nLen = 0; 
  
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);        // Disable the Watchdog timer resets
   pADI_CLKCTL->CLKCON5 = 0;                      // Enable clock to all peripherals
   
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
	 DioPul(pADI_GP0,0xFF);           // Enable pull-ups
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   DioOenPin(pADI_GP2,PIN4,1);                    // Configure P2.4 as an output - to toggle LED 
   DioSet(pADI_GP2, BIT4);

   NVIC_EnableIRQ(DMA_UART_TX_IRQn);                     // UArt Tx DMA interrupt enable
   NVIC_EnableIRQ(DMA_UART_RX_IRQn);                     // UArt Tx DMA interrupt enable

   EiCfg(EXTINT0, INT_EN, INT_FALL);                     // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                           // Enable External IRQ0 interrupt source in NVIC	 

   DMAINIT();    
   UartSetup();
   // Call function to initialize UART
   sprintf ( (char*)szTemp, "ADuCM322 DMA UART Demo \r\n" );   // Scan string 
   nLen = strlen((char*)szTemp);                               // Call function to calcualte the length of scanned string
   DmaPeripheralStructSetup(UARTTX_C,DMA_DSTINC_NO|				
      DMA_SRCINC_BYTE|DMA_SIZE_BYTE|DMA_BASIC);  		
   DmaGo(UARTTX_C,nLen,DMA_BASIC);                             // Setup SPI1TX structure chnl_cfg fields and number of values    
   DmaStructPtrOutSetup(UARTTX_C,nLen,(unsigned char*)szTemp); // Setup source/destination pointer fields

   DmaSet(0,UARTTX_B,0,0);
   UrtIntCfg(pADI_UART,COMIEN_EDMAT);


 while (1)
   {
	if (ucButtonPress == 1)
        {
          TestButtonPress();
          ucButtonPress = 0;
        }
   }
	 
}


void DMAINIT(void)  
{
   DmaBase(); 
}


// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
}
void TestButtonPress(void)
{
   unsigned char nLen = 0;  
   sprintf ( (char*)szTemp, "Button Pressed \r\n" );// Scan string 
   nLen = strlen((char*)szTemp);                  // Call function to calcualte the length of scanned string
   
   DmaPeripheralStructSetup(UARTTX_C,DMA_DSTINC_NO|				//
      DMA_SRCINC_BYTE|DMA_SIZE_BYTE|DMA_BASIC);  		

   DmaGo(UARTTX_C,nLen,DMA_BASIC);       // Setup SPI1TX structure chnl_cfg fields and number of values    
   DmaStructPtrOutSetup(UARTTX_C,nLen,(unsigned char*)szTemp);   // Setup source/destination pointer fields
   DmaSet(0,UARTTX_B,0,0);
   UrtIntCfg(pADI_UART,COMIEN_EDMAT);
}

void Ext_Int0_Handler()
{
  
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source
   ucButtonPress = 1;
}

void DMA_UART_TX_Int_Handler()      //mean DMA_SPI done
{
    ucIrqCnt2++;
	 UrtIntCfg(pADI_UART,COMIEN_EDMAT_DIS);
	 DmaClr (0,UARTTX_B,0,0);
	 ucTxBufferEmpty=1;
}

void DMA_UART_RX_Int_Handler()      //mean DMA_SPI done
{
   ucIrqCnt1++;
   UrtIntCfg(pADI_UART,~COMIEN_EDMAR);
}

// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




