/**
 *****************************************************************************
   @example  WDT.c
   @brief    Demonstrate the use of the watchdog timer.
             The watchdog timer is reset periodically so
             so that a reset does not occur.
             When external interrupt 0 is triggered the
             LED is turned ON and the watchdog is no longer
             reset, causing the part to reset and turn off the
             LED.
             
   @version  V0.4
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.3, May 2013: EIRQ0 is cleared inside the interrupt handler
   - V0.4, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <DeviceHeader.h>

#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>

int kickWDT = 1;
int cleared = 0;

void delay(long int);

int main (void)
{
   int i,j,k;
   DioOenPin(pADI_GP2, PIN4, 1);
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   WdtLd(0x200);
   WdtCfg(T3CON_MOD_PERIODIC,T3CON_PRE_DIV256,T3CON_IRQ_DIS,T3CON_PMD_DIS);
   EiCfg(EXTINT0, INT_EN, INT_FALL);
   NVIC_EnableIRQ(EINT0_IRQn);

	DioOenPin(pADI_GP2,PIN4,1);
   DioSet(pADI_GP2, BIT4);

   delay(1000);
	
   while (1)
   {
      if(kickWDT){
         while(!cleared){
            cleared = WdtClrInt();
         }
         cleared = 0;
      }
      delay(1000000);
	}
}

void WDog_Tmr_Int_Handler(void)
{

}

void Ext_Int0_Handler ()
{           
   kickWDT = 0;
   DioClr(pADI_GP2, BIT4);
   EiClr(EXTINT0);                  // Clear the interrupt0 source
}

// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




