/**
 *****************************************************************************
   @example  WakeupTimer.c
   @brief    Demonstrate the use of the Wakeup timer.
             
             
   @version  V0.2
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, May 2015: Initial Release
   - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <DeviceHeader.h>

#include <WutLib.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>

void delay(long int);

int main (void)
{
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);        // Disable the Watchdog timer resets
   DioOenPin(pADI_GP2, PIN4, 1);
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   pADI_CLKCTL->CLKCON5 = 0x00;                          // Enable clock to all peripherals
   pADI_CLKCTL->CLKCON1 = 0x200;                         // PCLK 20MHz, HCLK 80MHz
   DioOenPin(pADI_GP0,PIN3,1);
   //Timer 4 setup
   WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,0,0);             // T4 config, PCLK/4, periodic mode, PCLK source
 //  WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,0,0x200);           // T4 config, PCLK/1, periodic mode, LFOSC source
 //  WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,0,0x400);           // T4 config, PCLK/1, periodic mode, LFOSC source
 // WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,0,0x600);           // T4 config, PCLK/1, periodic mode, P1.0 source
 //  WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,1,0);             // T4 config, PCLK/16, periodic mode,PCLK source
   //WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,2,0);           // T4 config, PCLK/256, periodic mode,PCLK source
//   WutCfg(T4CON_MOD_PERIODIC,T4CON_WUEN_DIS,3,0);           // T4 config, PCLK/32768, periodic mode,PCLK source    
   WutCfgInt(T4IEN_WUFD|T4IEN_ROLL,1);                   // Interrupt on compare to D register and rollover
   WutLdWr(3,5000);                                      // Set the timeout period for 1mS
   WutGo(T4CON_ENABLE_EN);                               // Start wake-up timer
   NVIC_EnableIRQ(WUT_IRQn);                             // Enable Timer4 IRQ
   DioCfg(pADI_GP1,0x2);                                 // Set P1.0 as a clock input
   DioOenPin(pADI_GP2,PIN4,1);                           // Configure p2.4 as an output
   
   DioSet(pADI_GP2, BIT4);

   delay(1000);
	
   while (1)
   {
        pADI_GP0->GPTGL = 0x8;                           // Toggle P0.3;
	}
}

void WDog_Tmr_Int_Handler(void)
{

}
void WakeUp_Int_Handler(void)
{
   unsigned int iStatus = 0;
   iStatus = WutSta();
   if ((iStatus & 0x10) == 0x10)                          // Rollover
   {
      WutClrInt(T4CLRI_ROLL);                              // Clear wakeup rollover interrupt
   }
   if ((iStatus & 0x8) == 0x8)                          // Compare to Reg D
   {
      WutClrInt(T4CLRI_WUFD);                              // Clear wakeup for D register interrupt
   }
   if ((iStatus & 0x4) == 0x4)                          // Compare to Reg C
   {
      WutClrInt(T4CLRI_WUFC);                              // Clear wakeup for C register interrupt
   }
   if ((iStatus & 0x2) == 0x2)                          // Compare to Reg B
   {
      WutClrInt(T4CLRI_WUFB);                              // Clear wakeup for B register interrupt
   }
   if ((iStatus & 0x1) == 0x1)                          // Compare to Reg A
   {
      WutClrInt(T4CLRI_WUFA);                              // Clear wakeup for A register interrupt
   }
   pADI_GP2->GPTGL = 0x10;                               // Toggle P2.4;
}

// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




